#include <Wire.h>

#define REDPIN 11
#define YELLOWPIN 10
#define GREENPIN 9
#define AUDIOPIN 5

#define DELAYMSMAINLOOP 1

#define FASERUNS 10

#define ON 1
#define OFF 0

#define DEBUG 1
#define PRINT(x) Serial.print(x)


#define ADXL345_ADDRES 0x53

#define POWER_CTRL_REG 0x2D
#define ENABLE_3D_MEASURING 0x08

#define REG_X 0x32
#define REG_Y 0x34
#define REG_Z 0x36

#define BETA 0.9

#define DEVIATION 10

#define VIBE1 2
#define VIBE2 3

void setup() {

}


void InitLed(void) {
  pinMode(REDPIN, OUTPUT);
  pinMode(YELLOWPIN, OUTPUT);
  pinMode(GREENPIN, OUTPUT);
}


void InitAudio(void) {
  pinMode(AUDIOPIN, OUTPUT);
}


void InitSerial(int baud) {
  Serial.begin(baud);
}


void InitAccelero() {
  Wire.begin(); // Initiate the Wire library
  Wire.beginTransmission(ADXL345_ADDRES); // Start communicating with the device
  Wire.write(POWER_CTRL_REG); // Access/ talk to POWER_CTL Register - 0x2D
  Wire.write(ENABLE_3D_MEASURING); // (8dec -> 0000 1000 binary) Bit D3 High for measuring enable
  Wire.endTransmission();
  delay(10);
}

void InitVibe() {
  pinMode(VIBE1, OUTPUT);
  pinMode(VIBE2, OUTPUT);
}

int ReadXAxisADXL345(int addres) {
  Wire.beginTransmission(addres);
  Wire.write(REG_X);
  Wire.endTransmission(false);
  Wire.requestFrom(addres, 2, true);
  int out = ( Wire.read() | Wire.read() << 8);
  return out;
}


int ReadYAxisADXL345(int addres) {
  Wire.beginTransmission(addres);
  Wire.write(REG_Y);
  Wire.endTransmission(false);
  Wire.requestFrom(addres, 2, true);
  int out = ( Wire.read() | Wire.read() << 8);
  return out;
}


int ReadZAxisADXL345(int addres) {
  Wire.beginTransmission(addres);
  Wire.write(REG_Z);
  Wire.endTransmission(false);
  Wire.requestFrom(addres, 2, true);
  int out = ( Wire.read() | Wire.read() << 8);
  return out;
}


void ResetFase() {
  digitalWrite(REDPIN, LOW);
  digitalWrite(YELLOWPIN, LOW);
  digitalWrite(GREENPIN, LOW);
}

void ResetVibe() {
  digitalWrite(VIBE1, LOW);
  digitalWrite(VIBE2, LOW);
}


void ActiveteFase(int fase) {
  switch (fase) {
    case 1 :
      digitalWrite(REDPIN, HIGH);
      noTone(AUDIOPIN);
      ResetVibe();
      break;
    case 2 :
      digitalWrite(YELLOWPIN, HIGH);
      noTone(AUDIOPIN);
      break;
    case 3 :
      digitalWrite(GREENPIN, HIGH);
      noTone(AUDIOPIN);
      break;
    case 4 :
      tone(AUDIOPIN,500);
      break;
    case 5 :
      tone(AUDIOPIN,400);
      digitalWrite(VIBE1, HIGH);
      break;
    case 6 :
      tone(AUDIOPIN,300);
      
      break;
    case 7 :
      tone(AUDIOPIN,200);
      digitalWrite(VIBE2, HIGH);
      break;
    case 8 :
      tone(AUDIOPIN,100);
      break;
    default :
      ResetFase();
      noTone(AUDIOPIN);
      ResetVibe();
      break;
  }
}


void FaseRunner(int on) {
  static int loop_run = 0;
  static int was_on = 0;
  static int fase = 0;

  if (on) {
    if (was_on) {
      loop_run += 1;


      if (loop_run  >= FASERUNS) { // als gedefineerde aantal runs geweest zijn ophoog fase en voer fase
        fase += 1;
        loop_run = 0 ;
        ResetFase();
        ActiveteFase(fase);
      }

    } else { // Bij opnieuw beginnen alles reseten en in eerste fase starten
      was_on = 1;
      loop_run = 0 ;
      fase = 1;
      ResetFase();
      ActiveteFase(fase);
    }


  } else {
    if (was_on) { // Als aangeweest maar moet uit fase resetten
      ResetFase();
      was_on = 0 ;
    }
  }
}

void PrintAcellero(int x, int y, int z) {

  PRINT("X:");
  PRINT(x);
  PRINT(",\tY:");
  PRINT(y);
  PRINT(",\tZ:");
  PRINT(z);
  //PRINT("\n");
}

float moving_mean(float mean,int value, float beta){
  return((mean*beta)+((1-beta)*(float)value) );
}

float absolute(float value){
  if(value<0){
    value = value * -1;
  }
  return value;
}

void loop() {
  InitLed();
  InitAudio();
  InitSerial(9600);
  InitAccelero();
  InitVibe();

  int run_fases = 1;

  int x = 0 ;
  int y = 0 ;
  int z = 0 ;

  float x_mean =0 ,y_mean = 0 ,z_mean = 0;
  float x_diff,y_diff,z_diff = 0;

#if DEBUG
  PRINT("Startup\n");
#endif

  while (1) {

    delay(DELAYMSMAINLOOP);

    int active = 0;


      x= ReadXAxisADXL345(ADXL345_ADDRES);
      y= ReadYAxisADXL345(ADXL345_ADDRES);
      z= ReadZAxisADXL345(ADXL345_ADDRES);

      x_mean = moving_mean(x_mean,x,BETA);
      y_mean = moving_mean(y_mean,y,BETA);
      z_mean = moving_mean(z_mean,z,BETA);

      x_diff = absolute(x - x_mean);
      y_diff = absolute(y - y_mean);
      z_diff = absolute(z - z_mean);
      
#if DEBUG
      PrintAcellero(x,y,z);
      PRINT(",\tx m:");
      PRINT(x_mean);
      PRINT(",\ty m:");
      PRINT(y_mean);
      PRINT(",\tz m:");
      PRINT(z_mean);
      PRINT(",\tx d:");
      PRINT(x_diff);
      PRINT(",\ty d:");
      PRINT(y_diff);
      PRINT(",\tz d:");
      PRINT(z_diff);

      if (x_diff>DEVIATION){
        PRINT(",\t Trigger X");
        active++;
      }

      if (y_diff>DEVIATION){
        PRINT(",\t Trigger Y");
        active++;
      }

      if (z_diff>DEVIATION){
        PRINT(",\t Trigger Z");
        active++;
      }
      
      PRINT("\n");
#endif
      if(!active){
        FaseRunner(ON);
      } else {
        FaseRunner(OFF);
      }

  }

}
